VERSION 5.00
Begin VB.Form frmMain 
   BorderStyle     =   3  'Fixed Dialog
   Caption         =   "DirectX-Graphics: Lesson 03"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4680
   ControlBox      =   0   'False
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   213
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   312
   ShowInTaskbar   =   0   'False
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'###############################
'
'           Title: DirectX-Graphics Article 03
'           Desc: The fun stuff! Textures, Models and lighting.
'           Written by: Jack Hoxley
'           Date: November 17th 2001
'           Contact: Jollyjeffers@Greenonions.netscapeonline.co.uk
'                       www.vbexplorer.com/directx4vb/
'           NB: Sample/Article was written for GameDev.Net
'
'###############################

Dim Dx As DirectX8 'The master Object, everything comes from here
Dim D3D As Direct3D8 'This controls all things 3D
Dim D3DX As D3DX8   'a very useful set of helper functions
Dim D3DDevice As Direct3DDevice8 'This actually represents the hardware doing the rendering
Dim bRunning As Boolean 'Controls whether the program is running or not...
#Const USE_INTERNAL_MODEL = 0
#Const USE_D3D_LIGHTING = 1

Const FVF_TLVERTEX = (D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR)
Const FVF_LVERTEX = (D3DFVF_XYZ Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR Or D3DFVF_TEX1)
Const FVF_VERTEX = (D3DFVF_XYZ Or _
                    D3DFVF_NORMAL Or _
                    D3DFVF_TEX1)

Private Type TLVERTEX
    X As Single
    Y As Single
    Z As Single
    rhw As Single
    Color As Long
    Specular As Long
    tU As Single
    tV As Single
End Type

Private Type LITVERTEX
    X As Single
    Y As Single
    Z As Single
    Color As Long
    Specular As Long
    tU As Single
    tV As Single
End Type

Private Type VERTEX
    P As D3DVECTOR
    N As D3DVECTOR
    T As D3DVECTOR2
End Type

Dim CubeVerts(0 To 35) As LITVERTEX '//for our 3D cube

Dim matProj As D3DMATRIX
Dim matView As D3DMATRIX
Dim matWorld As D3DMATRIX

Const PI = 3.14159
Const RAD = PI / 180
Const DEG = 180 / PI


Private Declare Function GetTickCount Lib "kernel32" () As Long
Private FrameCount As Long
Private FrameRate As Long
Private FrameRateLastCheck As Long

Dim CubeTex As Direct3DTexture8 '//the object that stores the data for this texture.


Dim nMaterials As Long
Dim MeshMaterials() As D3DMATERIAL8
Dim MeshTextures() As Direct3DTexture8
Dim CubeMesh As D3DXMesh

Dim nMaterials2 As Long
Dim MeshMaterials2() As D3DMATERIAL8
Dim MeshTextures2() As Direct3DTexture8
Dim LightingMesh As D3DXMesh


'// Initialise : This procedure kick starts the whole process.
'// It'll return true for success, false if there was an error.
Public Function Initialise() As Boolean
On Error GoTo ErrHandler:

Dim DispMode As D3DDISPLAYMODE '//Describes our Display Mode
Dim tmpDispMode As D3DDISPLAYMODE '//used during the enumeration of avail. modes
Dim D3DWindow As D3DPRESENT_PARAMETERS '//Describes our Viewport
Dim i As Long '//so we can loop through the avail. display modes
Dim BehaviourFlag As Long '//for the CreateDevice call.

Set Dx = New DirectX8  '//Create our Master Object
Set D3D = Dx.Direct3DCreate() '//Let our Master Object create the Direct3D Interface
Set D3DX = New D3DX8

'##################
'##  FULLSCREEN MODE ##
'#################

DispMode.Format = CheckDisplayMode(640, 480, 32)
Debug.Print "CHECKDISP(640,480,32) = ", DispMode.Format, D3DFMT_UNKNOWN
If DispMode.Format > D3DFMT_UNKNOWN Then
    '640x480x32 is supported
    Debug.Print "USING 640x480x32 Display Mode"
    DispMode.Width = 640: DispMode.Height = 480
Else
    DispMode.Format = CheckDisplayMode(640, 480, 16)
    If DispMode.Format > D3DFMT_UNKNOWN Then
        '640x480x16 is supported
        Debug.Print "USING 640x480x16 Display Mode"
        DispMode.Width = 640: DispMode.Height = 480
    Else
        'hmm, neither are supported. oh well...
        MsgBox "Your hardware does not appear to support" _
                     & " 640x480 display modes in either 16 bit or 32 bit modes. Exiting" _
                    , vbInformation, "Error"
        Unload Me
        End
        
    End If
End If

D3DWindow.BackBufferCount = 1
D3DWindow.BackBufferFormat = DispMode.Format
D3DWindow.BackBufferWidth = DispMode.Width
D3DWindow.BackBufferHeight = DispMode.Height
D3DWindow.hDeviceWindow = frmMain.hWnd
D3DWindow.AutoDepthStencilFormat = D3DFMT_D16
D3DWindow.EnableAutoDepthStencil = 1
D3DWindow.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC

'############################
'## CHECK THE DEVICE CAPABILITIES ##
'###########################
Dim DevCaps As D3DCAPS8
On Local Error Resume Next
    D3D.GetDeviceCaps D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DevCaps
If Err.Number = D3DERR_INVALIDDEVICE Then
    'We couldn't get data from the hardware device - probably doesn't exist...
    D3D.GetDeviceCaps D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, DevCaps
    Err.Clear '//Remove the error value..
End If

Debug.Print "This device supports a maximum of " & DevCaps.MaxActiveLights & " Active lights"


'//For Hardware vertex processing:
If (DevCaps.DevCaps And D3DDEVCAPS_HWTRANSFORMANDLIGHT) Then
    Debug.Print "Hardware Transform and lighting supported"
    'D3DCREATE_HARDWARE_VERTEXPROCESSING should be used in the CreateDevice call
    BehaviourFlag = D3DCREATE_HARDWARE_VERTEXPROCESSING 'the default
Else
    Debug.Print "Hardware Transform and lighting is not supported"
    BehaviourFlag = D3DCREATE_SOFTWARE_VERTEXPROCESSING 'the default
End If

'//For Pure Device processing:
If (DevCaps.DevCaps And D3DDEVCAPS_PUREDEVICE) Then
    Debug.Print "Pure Device is supported."
    'D3DCREATE_PUREDEVICE should be used in the CreateDevice call
    BehaviourFlag = D3DCREATE_PUREDEVICE 'the default
Else
    Debug.Print "Pure device is not supported"
End If

Debug.Print "MAX TEX SIZE: ", DevCaps.MaxTextureWidth & "x" & DevCaps.MaxTextureHeight

'//This line creates a device that uses a hardware device if possible; software vertex processing and uses the form as it's target
'//See the lesson text for more information on this line...
Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                                                    frmMain.hWnd, _
                                                    BehaviourFlag, _
                                                        D3DWindow)
                                                        
If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                         D3DWindow.BackBufferFormat, 0, D3DRTYPE_TEXTURE, _
                         D3DFMT_X8R8G8B8) = D3D_OK Then
                         
    Debug.Print "32 Bit textures with no alpha are supported"
End If

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                         D3DWindow.BackBufferFormat, 0, D3DRTYPE_TEXTURE, _
                         D3DFMT_A8R8G8B8) = D3D_OK Then
                         
    Debug.Print "32 Bit textures with alpha are supported"
End If

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                         D3DWindow.BackBufferFormat, 0, D3DRTYPE_TEXTURE, _
                         D3DFMT_X1R5G5B5) = D3D_OK Then
                         
    Debug.Print "16 Bit (555 format) textures with no alpha are supported"
End If

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                         D3DWindow.BackBufferFormat, 0, D3DRTYPE_TEXTURE, _
                         D3DFMT_R5G6B5) = D3D_OK Then
                         
    Debug.Print "16 Bit (565) textures with no alpha are supported"
End If

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, _
                         D3DWindow.BackBufferFormat, 0, D3DRTYPE_TEXTURE, _
                         D3DFMT_A1R5G5B5) = D3D_OK Then
                         
    Debug.Print "16 Bit textures with 1-bit alpha are supported"
End If

                                                        
'//Configure the rendering device

    
#If USE_D3D_LIGHTING And (USE_INTERNAL_MODEL = 0) Then
    D3DDevice.SetVertexShader FVF_VERTEX '//Tell it what type of vertex we are using
    D3DDevice.SetRenderState D3DRS_LIGHTING, 1   '//Disable lighting.
    D3DDevice.SetRenderState D3DRS_AMBIENT, D3DColorXRGB(50, 50, 50)
    D3DDevice.SetRenderState D3DRS_ZENABLE, 1
    D3DDevice.SetRenderState D3DRS_CULLMODE, D3DCULL_NONE
#Else
    D3DDevice.SetVertexShader FVF_LVERTEX '//Tell it what type of vertex we are using
    D3DDevice.SetRenderState D3DRS_LIGHTING, 0  '//Disable lighting.
    D3DDevice.SetRenderState D3DRS_ZENABLE, 1
    D3DDevice.SetRenderState D3DRS_CULLMODE, D3DCULL_NONE
#End If
D3DDevice.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR
D3DDevice.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR

'D3DDevice.SetRenderState D3DRS_FILLMODE, D3DFILL_WIREFRAME  'use this line to render wireframe
D3DDevice.SetRenderState D3DRS_FILLMODE, D3DFILL_SOLID  'use this line to render normally

'//configure the world matrices

'//1. The World Matrix
D3DXMatrixIdentity matWorld
D3DDevice.SetTransform D3DTS_WORLD, matWorld 'commit this matrix to the device

'//2. The View Matrix
D3DXMatrixLookAtLH matView, MakeVector(0, 15, -10), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
D3DDevice.SetTransform D3DTS_VIEW, matView

'//3. The projection Matrix
D3DXMatrixPerspectiveFovLH matProj, PI / 3, 1, 0.1, 75
D3DDevice.SetTransform D3DTS_PROJECTION, matProj

Set CubeTex = D3DX.CreateTextureFromFileEx(D3DDevice, _
                        App.Path & "\crate_tex.jpg", _
                        256, 256, 1, 0, _
                        DispMode.Format, D3DPOOL_MANAGED, _
                        D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, _
                        0, ByVal 0, ByVal 0)
If CubeTex Is Nothing Then Debug.Print "CubeTex was not loaded correctly."
                        
#If USE_D3D_LIGHTING And (USE_INTERNAL_MODEL = 0) Then
    Dim Light As D3DLIGHT8
    Light.Type = D3DLIGHT_DIRECTIONAL
    Light.position = MakeVector(0, -1, 0)
    Light.Direction = MakeVector(0, -1, 0)
    Light.Range = 17
    Light.diffuse.R = 0
    Light.diffuse.G = 0
    Light.diffuse.B = 255
    D3DDevice.SetLight 0, Light
    D3DDevice.LightEnable 0, 1
    
    SetupLighting
#End If

InitialiseGeometry '//Setup the vertices

Initialise = True '//We succeeded
Exit Function

ErrHandler:
Debug.Print "Error Number Returned: " & Err.Number, Err.Description
Initialise = False
End Function

Public Sub Render()

Dim i As Long
'//1. We need to clear the render device before we can draw anything
'       This must always happen before you start rendering stuff...
D3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET Or D3DCLEAR_ZBUFFER, &H8080CC, 1#, 0
'the hexidecimal value in the middle is the same as when you're using colours in HTML - if you're familiar
'with that.

'//2. Next we would render everything.

D3DDevice.BeginScene
    
    #If USE_INTERNAL_MODEL = 1 Then
        
        D3DDevice.SetTexture 0, CubeTex
        D3DDevice.DrawPrimitiveUP D3DPT_TRIANGLELIST, 12, CubeVerts(0), Len(CubeVerts(0))
    
    #ElseIf (USE_INTERNAL_MODEL = 0) And (USE_D3D_LIGHTING = 0) Then
        For i = 0 To nMaterials - 1
            D3DDevice.SetTexture 0, MeshTextures(i)
            D3DDevice.SetMaterial MeshMaterials(i)
            CubeMesh.DrawSubset i
        Next i
        
    #ElseIf USE_D3D_LIGHTING And (USE_INTERNAL_MODEL = 0) Then
        For i = 0 To nMaterials2 - 1
            D3DDevice.SetTexture 0, MeshTextures2(i)
            D3DDevice.SetMaterial MeshMaterials2(i)
            LightingMesh.DrawSubset i
        Next i
    #End If

D3DDevice.EndScene

'//3. Update the frame to the screen...
'       This is the same as the Primary.Flip method as used in DirectX 7
'       These values below should work for almost all cases...
D3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

Private Sub InitialiseGeometry()
On Error GoTo BailOut:

'//0. Any variables required
    Dim mtrlBuffer As D3DXBuffer '//Holds some useful data for us...
    Dim i As Long 'Loop variable
    Dim TextureFile As String 'the texture required

#If USE_INTERNAL_MODEL = 1 Then
'//1. Define the colours at each corner
    Const Corner000 As Long = &HFF0000    'red                  0
    Const Corner001 As Long = &HFF00       'green               1
    Const Corner010 As Long = &HFF           'blue                 2
    Const Corner011 As Long = &HFF00FF    'magenta           3
    Const Corner100 As Long = &HFFFF00    'yellow               4
    Const Corner101 As Long = &HFFFF       'cyan                 5
    Const Corner110 As Long = &HFF8000   'orange              6
    Const Corner111 As Long = &HFFFFFF       'white              7
    
    
    
'//2. Define the faces
    'top
        CubeVerts(0) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(1) = CreateLitVertex(1, 1, -1, Corner110, 0, 1, 0)
        CubeVerts(2) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 1)
        
        CubeVerts(3) = CreateLitVertex(1, 1, -1, Corner110, 0, 1, 0)
        CubeVerts(4) = CreateLitVertex(1, 1, 1, Corner111, 0, 1, 1)
        CubeVerts(5) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 1)
        
    'bottom
        CubeVerts(6) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 0)
        CubeVerts(7) = CreateLitVertex(1, -1, -1, Corner100, 0, 1, 0)
        CubeVerts(8) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 1)
        
        CubeVerts(9) = CreateLitVertex(1, -1, -1, Corner100, 0, 1, 0)
        CubeVerts(10) = CreateLitVertex(1, -1, 1, Corner101, 0, 1, 1)
        CubeVerts(11) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 1)
    
    'left
        CubeVerts(12) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(13) = CreateLitVertex(-1, 1, 1, Corner011, 0, 1, 0)
        CubeVerts(14) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 1)
        
        CubeVerts(15) = CreateLitVertex(-1, 1, 1, Corner011, 0, 1, 0)
        CubeVerts(16) = CreateLitVertex(-1, -1, 1, Corner001, 0, 1, 1)
        CubeVerts(17) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 1)
    
    'right
        CubeVerts(18) = CreateLitVertex(1, 1, -1, Corner110, 0, 0, 0)
        CubeVerts(19) = CreateLitVertex(1, 1, 1, Corner111, 0, 1, 0)
        CubeVerts(20) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 1)
        
        CubeVerts(21) = CreateLitVertex(1, 1, 1, Corner111, 0, 1, 0)
        CubeVerts(22) = CreateLitVertex(1, -1, 1, Corner101, 0, 1, 1)
        CubeVerts(23) = CreateLitVertex(1, -1, -1, Corner100, 0, 0, 1)
    
    'front
        CubeVerts(24) = CreateLitVertex(-1, 1, 1, Corner011, 0, 0, 0)
        CubeVerts(25) = CreateLitVertex(1, 1, 1, Corner111, 0, 1, 0)
        CubeVerts(26) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 1)
        
        CubeVerts(27) = CreateLitVertex(1, 1, 1, Corner111, 0, 1, 0)
        CubeVerts(28) = CreateLitVertex(1, -1, 1, Corner101, 0, 1, 1)
        CubeVerts(29) = CreateLitVertex(-1, -1, 1, Corner001, 0, 0, 1)
    
    'back
        CubeVerts(30) = CreateLitVertex(-1, 1, -1, Corner010, 0, 0, 0)
        CubeVerts(31) = CreateLitVertex(1, 1, -1, Corner110, 0, 1, 0)
        CubeVerts(32) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 1)
        
        CubeVerts(33) = CreateLitVertex(1, 1, -1, Corner110, 0, 1, 0)
        CubeVerts(34) = CreateLitVertex(1, -1, -1, Corner100, 0, 1, 1)
        CubeVerts(35) = CreateLitVertex(-1, -1, -1, Corner000, 0, 0, 1)
#Else

'//3. Get the data from the file
    Set CubeMesh = D3DX.LoadMeshFromX(App.Path & "\cube_3d.x", D3DXMESH_MANAGED, D3DDevice, Nothing, mtrlBuffer, nMaterials)

    If CubeMesh Is Nothing Then GoTo BailOut: '//Dont continue if the above call did not work
    
'//4. Allocate the space required for the materials and textures used:
    ReDim MeshMaterials(nMaterials) As D3DMATERIAL8
    ReDim MeshTextures(nMaterials) As Direct3DTexture8
    
'//5. Now we fill our arrays with the information required
    For i = 0 To nMaterials - 1
    
        '//Get D3DX to copy the data that we loaded from the file into our structure
        D3DX.BufferGetMaterial mtrlBuffer, i, MeshMaterials(i)
        
        '//Fill in the missing gaps - the Ambient properties
        MeshMaterials(i).Ambient = MeshMaterials(i).diffuse
        
        '//get the name of the texture used for this part of the mesh
        TextureFile = D3DX.BufferGetTextureName(mtrlBuffer, i)
        
        '//Now create the texture
        If TextureFile <> "" Then 'Dont try to create a texture from an empty string
            Set MeshTextures(i) = D3DX.CreateTextureFromFileEx(D3DDevice, App.Path & "\" & TextureFile, 256, 256, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, 0, ByVal 0, ByVal 0)
        End If
        
    Next i
    
'//6. Output some info to the debug window - not required, just interesting
    Debug.Print "Number of Faces in mesh: " & CubeMesh.GetNumFaces
    Debug.Print "Number of Vertices in mesh: " & CubeMesh.GetNumVertices
    Debug.Print "Number of segments in mesh: " & nMaterials
    Debug.Print "Cube Mesh FVF: " & CubeMesh.GetFVF
    

'//7. Get the data from the file
    Set LightingMesh = D3DX.LoadMeshFromX(App.Path & "\Light_Scene.x", D3DXMESH_MANAGED, D3DDevice, Nothing, mtrlBuffer, nMaterials2)

    If LightingMesh Is Nothing Then GoTo BailOut: '//Dont continue if the above call did not work
    
'//8. Allocate the space required for the materials and textures used:
    ReDim MeshMaterials2(nMaterials2) As D3DMATERIAL8
    ReDim MeshTextures2(nMaterials2) As Direct3DTexture8
    
'//9. Now we fill our arrays with the information required
    For i = 0 To nMaterials2 - 1
    
        '//Get D3DX to copy the data that we loaded from the file into our structure
        D3DX.BufferGetMaterial mtrlBuffer, i, MeshMaterials2(i)
        
        '//Fill in the missing gaps - the Ambient properties
        MeshMaterials2(i).Ambient = MeshMaterials2(i).diffuse
        
        '//get the name of the texture used for this part of the mesh
        TextureFile = D3DX.BufferGetTextureName(mtrlBuffer, i)
        
        '//Now create the texture
        If TextureFile <> "" Then 'Dont try to create a texture from an empty string
            Set MeshTextures2(i) = D3DX.CreateTextureFromFileEx(D3DDevice, App.Path & "\" & TextureFile, 256, 256, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, 0, ByVal 0, ByVal 0)
        End If
        
    Next i
    
'//10. Output some info to the debug window - not required, just interesting
    Debug.Print "Number of Faces in mesh: " & LightingMesh.GetNumFaces
    Debug.Print "Number of Vertices in mesh: " & LightingMesh.GetNumVertices
    Debug.Print "Number of segments in mesh: " & nMaterials2
    Debug.Print "Lighting Mesh FVF: " & LightingMesh.GetFVF
    
#End If
Exit Sub
BailOut:
    Debug.Print "An error occured creating the geometry: ", Err.Number, Err.Description, D3DX.GetErrorString(Err.Number)
End Sub

Private Function CreateLitVertex(X As Single, Y As Single, Z As Single, Colour As Long, Specular As Long, tU As Single, tV As Single) As LITVERTEX
    CreateLitVertex.X = X
    CreateLitVertex.Y = Y
    CreateLitVertex.Z = Z
    CreateLitVertex.Color = Colour
    CreateLitVertex.Specular = Specular
    CreateLitVertex.tU = tU
    CreateLitVertex.tV = tV
End Function

Private Function MakeVector(X As Single, Y As Single, Z As Single) As D3DVECTOR
    MakeVector.X = X: MakeVector.Y = Y: MakeVector.Z = Z
End Function

Private Function CheckDisplayMode(Width As Long, Height As Long, Depth As Long) As CONST_D3DFORMAT
'NB: I changed this code slightly from the original part-2 version; the OR operators were set up wrong. whoops.
'//0. any variables
    Dim i As Long
    Dim DispMode As D3DDISPLAYMODE
    
'//1. Scan through
    For i = 0 To D3D.GetAdapterModeCount(0) - 1
        D3D.EnumAdapterModes 0, i, DispMode
        If DispMode.Width = Width Then
            If DispMode.Height = Height Then
                If (DispMode.Format = D3DFMT_R5G6B5) Or (DispMode.Format = D3DFMT_X1R5G5B5) Or (DispMode.Format = D3DFMT_X4R4G4B4) Then
                    '16 bit mode
                        If Depth = 16 Then
                            CheckDisplayMode = DispMode.Format
                            Exit Function
                        End If
                ElseIf (DispMode.Format = D3DFMT_R8G8B8) Or (DispMode.Format = D3DFMT_X8R8G8B8) Then
                    '32bit mode
                        If Depth = 32 Then
                            CheckDisplayMode = DispMode.Format
                            Exit Function
                        End If
                End If
            End If
        End If
    Next i
CheckDisplayMode = D3DFMT_UNKNOWN
End Function

Private Function GetNormal(v0 As D3DVECTOR, v1 As D3DVECTOR, v2 As D3DVECTOR) As D3DVECTOR
    '//0. Any Variables
        Dim v01 As D3DVECTOR, v02 As D3DVECTOR, vNorm As D3DVECTOR
    
    '//1. Get the vectors 0->1 and 0->2
        D3DXVec3Subtract v01, v1, v0
        D3DXVec3Subtract v02, v2, v0
    
    '//2. Get the cross product
        D3DXVec3Cross vNorm, v01, v02
    
    '//3. Normalize this vector
        D3DXVec3Normalize vNorm, vNorm
        
    '//4. Return the value:
        GetNormal = vNorm
End Function

Private Function GetNormal2(v0 As D3DVECTOR, v1 As D3DVECTOR, v2 As D3DVECTOR) As D3DVECTOR
    '//0. Any Variables
        Dim L As Double
        Dim v01 As D3DVECTOR, v02 As D3DVECTOR, vNorm As D3DVECTOR
    
    '//1. Get the vectors 0->1 and 0->2
        v01.X = v1.X - v0.X
        v01.Y = v1.Y - v0.Y
        v01.Z = v1.Z - v0.Z
        
        v02.X = v2.X - v0.X
        v02.Y = v2.Y - v0.Y
        v02.Z = v2.Z - v0.Z
    
    '//2. Get the cross product
        vNorm.X = (v01.Y * v02.Z) - (v01.Z * v02.Y)
        vNorm.X = (v01.Z * v02.X) - (v01.X * v02.Z)
        vNorm.X = (v01.X * v02.Y) - (v01.Y * v02.X)
    
    '//3. Normalize this vector
        L = Sqr((vNorm.X * vNorm.X) + (vNorm.Y * vNorm.Y) + (vNorm.Z * vNorm.Z))
        vNorm.X = vNorm.X / L
        vNorm.Y = vNorm.Y / L
        vNorm.Z = vNorm.Z / L
        
    '//4. Return the value:
        GetNormal2 = vNorm
End Function

Private Function CreateD3DColorVal(A As Single, R As Single, G As Single, B As Single) As D3DCOLORVALUE
    CreateD3DColorVal.A = A
    CreateD3DColorVal.R = R
    CreateD3DColorVal.G = G
    CreateD3DColorVal.B = B
End Function

Private Sub SetupLighting()

Dim lghtPoint As D3DLIGHT8

lghtPoint.Type = D3DLIGHT_POINT
lghtPoint.diffuse = CreateD3DColorVal(1, 1, 0, 0)
lghtPoint.position = MakeVector(0, 0, -10)
lghtPoint.Range = 25#
lghtPoint.Attenuation0 = 0#
lghtPoint.Attenuation1 = 1#
lghtPoint.Attenuation2 = 0#

Dim lghtDirectional As D3DLIGHT8

lghtDirectional.Type = D3DLIGHT_DIRECTIONAL
lghtDirectional.Direction = MakeVector(0, -1, 0)
lghtDirectional.position = MakeVector(1, 1, 1) 'shouldn't be left as 0
lghtDirectional.Range = 1 'shouldn't be left as 0
lghtDirectional.diffuse = CreateD3DColorVal(1, 0, 1, 0) 'green light

Dim lghtSpot As D3DLIGHT8

lghtSpot.Type = D3DLIGHT_SPOT
lghtSpot.Range = 50#
lghtSpot.diffuse = CreateD3DColorVal(1, 0, 0, 1)
lghtSpot.Direction = MakeVector(0, 0, 1)
lghtSpot.position = MakeVector(0, 0, -20)
lghtSpot.Theta = 0.05 * PI
lghtSpot.Phi = 0.1 * PI
lghtSpot.Attenuation0 = 0.1
lghtSpot.Attenuation1 = 1#
lghtSpot.Attenuation2 = 0#

D3DDevice.SetLight 0, lghtPoint
D3DDevice.SetLight 1, lghtDirectional
D3DDevice.SetLight 2, lghtSpot

D3DDevice.LightEnable 0, 1
D3DDevice.LightEnable 1, 1
D3DDevice.LightEnable 2, 1
End Sub


Private Sub Form_Click()
bRunning = False
End Sub

Private Sub Form_KeyUp(KeyCode As Integer, Shift As Integer)
Select Case KeyCode
    Case vbKey1 'Point only
        D3DDevice.LightEnable 0, 1
        D3DDevice.LightEnable 1, 0
        D3DDevice.LightEnable 2, 0
        D3DDevice.SetRenderState D3DRS_AMBIENT, D3DColorXRGB(0, 0, 0)
    Case vbKey2 'Directional Only
        D3DDevice.LightEnable 0, 0
        D3DDevice.LightEnable 1, 1
        D3DDevice.LightEnable 2, 0
        D3DDevice.SetRenderState D3DRS_AMBIENT, D3DColorXRGB(0, 0, 0)
    Case vbKey3 'Spot light only
        D3DDevice.LightEnable 0, 0
        D3DDevice.LightEnable 1, 0
        D3DDevice.LightEnable 2, 1
        D3DDevice.SetRenderState D3DRS_AMBIENT, D3DColorXRGB(0, 0, 0)
    Case vbKey4 'Ambient only
        D3DDevice.LightEnable 0, 0
        D3DDevice.LightEnable 1, 0
        D3DDevice.LightEnable 2, 0
        D3DDevice.SetRenderState D3DRS_AMBIENT, D3DColorXRGB(255, 255, 255)
    Case vbKeyW
        D3DDevice.SetRenderState D3DRS_FILLMODE, D3DFILL_WIREFRAME
        
    Case vbKeyS
        D3DDevice.SetRenderState D3DRS_FILLMODE, D3DFILL_SOLID
End Select

End Sub

Private Sub Form_Load()
Dim matTemp As D3DMATRIX
Dim Angle As Single
Dim LastUpdated As Long
Const Degrees_Per_Second As Single = 90#

LastUpdated = GetTickCount()
FrameRateLastCheck = GetTickCount()

Me.Show '//Make sure our window is visible

bRunning = Initialise()
Debug.Print "Device Creation Return Code : ", bRunning 'So you can see what happens...

Do While bRunning
    D3DXMatrixIdentity matWorld
    
    #If USE_INTERNAL_MODEL = 1 Then
        D3DXMatrixIdentity matTemp
        D3DXMatrixRotationX matTemp, Angle * RAD
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
        D3DXMatrixIdentity matTemp
        D3DXMatrixRotationY matTemp, Angle * RAD
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
        D3DXMatrixIdentity matTemp
        D3DXMatrixScaling matTemp, 1.5, 1.5, 1.5
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
    #ElseIf (USE_INTERNAL_MODEL = 0) And USE_D3D_LIGHTING Then
        D3DXMatrixIdentity matTemp
        D3DXMatrixRotationX matTemp, Angle * RAD
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
        'D3DXMatrixIdentity matTemp
        'D3DXMatrixRotationY matTemp, Angle * RAD
        'D3DXMatrixMultiply matWorld, matWorld, matTemp
    
        D3DXMatrixIdentity matTemp
        D3DXMatrixScaling matTemp, 0.025, 0.025, 0.025
        D3DXMatrixMultiply matWorld, matWorld, matTemp
    #ElseIf (USE_INTERNAL_MODEL = 0) And (USE_D3D_LIGHTING = 0) Then
        D3DXMatrixIdentity matTemp
        D3DXMatrixRotationX matTemp, Angle * RAD
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
        D3DXMatrixIdentity matTemp
        D3DXMatrixRotationY matTemp, Angle * RAD
        D3DXMatrixMultiply matWorld, matWorld, matTemp
        
        D3DXMatrixIdentity matTemp
        D3DXMatrixScaling matTemp, 0.5, 0.5, 0.5
        D3DXMatrixMultiply matWorld, matWorld, matTemp
    #End If
    
    Angle = Angle + (Degrees_Per_Second * ((GetTickCount() - LastUpdated) / 1000))
    If Angle > 360 Then Angle = Angle - 360
    LastUpdated = GetTickCount()
    
    D3DDevice.SetTransform D3DTS_WORLD, matWorld
    
    Render '//Update the frame...
    
    FrameCount = FrameCount + 1
    If (GetTickCount() - FrameRateLastCheck >= 1000) Then
        'one second has passed
        FrameRate = FrameCount
        FrameCount = 0
        FrameRateLastCheck = GetTickCount()
        Debug.Print FrameRate & "fps"
    End If
    DoEvents '//Allow windows time to think; otherwise you'll get into a really tight (and bad) loop...

Loop '//Begin the next frame...

'//If we've gotten to this point the loop must have been terminated
'   So we need to clean up after ourselves. This isn't essential, but it'
'   good coding practise.

On Error Resume Next 'If the objects were never created;
'                               (the initialisation failed) we might get an
'                               error when freeing them... which we need to
'                               handle, but as we're closing anyway...
Set D3DDevice = Nothing
Set D3D = Nothing
Set Dx = Nothing
Debug.Print "All Objects Destroyed"

'//Final termination:
Unload Me
End
End Sub

